package com.github.axet.hourlyreminder.fragments;

import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.graphics.drawable.StateListDrawable;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v7.preference.PreferenceManager;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.FrameLayout;
import android.widget.TextView;

import com.github.axet.androidlibrary.app.Storage;
import com.github.axet.androidlibrary.widgets.RingtoneChoicer;
import com.github.axet.androidlibrary.widgets.Toast;
import com.github.axet.hourlyreminder.R;
import com.github.axet.hourlyreminder.alarms.ReminderSet;
import com.github.axet.hourlyreminder.alarms.WeekSet;
import com.github.axet.hourlyreminder.app.HourlyApplication;
import com.github.axet.hourlyreminder.app.Sound;
import com.github.axet.hourlyreminder.dialogs.HoursDialogFragment;
import com.github.axet.hourlyreminder.dialogs.RepeatDialogFragment;

import java.io.File;
import java.util.ArrayList;

public class RemindersFragment extends WeekSetFragment implements DialogInterface.OnDismissListener {
    HourlyApplication.ItemsStorage items;

    public static String everyText(Context context, int repeat) {
        String str = "";
        if (repeat < 0) {
            str = ":" + (-repeat) + context.getString(R.string.min_symbol);
        } else {
            switch (repeat) {
                case 60:
                    str = "1" + context.getString(R.string.hour_symbol);
                    break;
                default:
                    str = "" + repeat + context.getString(R.string.min_symbol);
                    break;
            }
        }
        return str;
    }

    public static class ViewHolder extends WeekSetFragment.ViewHolder {
        public View every;
        public TextView everyText;

        public ViewHolder(View v) {
            super(v);
            every = v.findViewById(R.id.alarm_every);
            everyText = (TextView) v.findViewById(R.id.alarm_every_text);
        }
    }

    public RemindersFragment() {
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        items = HourlyApplication.from(getContext()).items;
        adapter = new Adapter<ViewHolder>() {
            @Override
            public int getItemCount() {
                return items.reminders.size();
            }

            @Override
            public Object getItem(int position) {
                return items.reminders.get(position);
            }

            @Override
            public long getItemId(int position) {
                return items.reminders.get(position).id;
            }

            @Override
            public int getPosition(long id) {
                for (int i = 0; i < items.reminders.size(); i++) {
                    if (items.reminders.get(i).id == id)
                        return i;
                }
                return -1;
            }

            @Override
            public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
                LayoutInflater inflater = LayoutInflater.from(parent.getContext());
                View convertView = inflater.inflate(R.layout.reminder, parent, false);
                return new ViewHolder(convertView);
            }

            @Override
            public void fillDetailed(final ViewHolder h, final WeekSet w, final boolean animate) {
                super.fillDetailed(h, w, animate);

                final ReminderSet rr = (ReminderSet) w;

                h.ringtoneButton.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        Uri uri = w.ringtoneValue;

                        if (uri == null)
                            uri = ReminderSet.DEFAULT_NOTIFICATION;

                        RingtoneChoicer r = new RingtoneChoicer() {
                            @Override
                            public void onResult(Uri uri, boolean tmp) {
                                if (tmp) {
                                    File f = storage.storeRingtone(uri);
                                    uri = Uri.fromFile(f);
                                }
                                w.ringtoneValue = fallbackUri(uri);
                                save(w);
                            }

                            @Override
                            public void onDismiss() {
                                choicer = null;
                            }
                        };
                        r.setPermissionsDialog(RemindersFragment.this, Storage.PERMISSIONS_RO, RESULT_RINGTONE);
                        r.setRingtone(RemindersFragment.this, ReminderSet.TYPE_NOTIFICATION, ReminderSet.DEFAULT_NOTIFICATION, getString(R.string.Reminder), RESULT_RINGTONE);
                        choicer = r;
                        choicer.show(uri);
                    }
                });

                updateTime(h, (ReminderSet) w);
                h.time.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        HoursDialogFragment dialog = new HoursDialogFragment();

                        Bundle args = new Bundle();
                        args.putInt("index", items.reminders.indexOf(rr));
                        args.putStringArrayList("hours", new ArrayList<>(rr.hours));
                        if (rr.start != null)
                            args.putString("start", rr.start.key);
                        if (rr.end != null)
                            args.putString("end", rr.end.key);
                        args.putInt("repeat", rr.repeat);

                        dialog.setArguments(args);
                        dialog.show(getFragmentManager(), "");
                    }
                });

                h.every.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        RepeatDialogFragment d = new RepeatDialogFragment();

                        Bundle args = new Bundle();
                        args.putInt("index", items.reminders.indexOf(w));
                        args.putInt("mins", rr.repeat);
                        d.setArguments(args);

                        d.show(getFragmentManager(), "");
                    }
                });
                updateEvery(h, w);

                h.weekdays.setButtonDrawable(new StateListDrawable());
                h.weekdays.setClickable(false);
            }

            @Override
            public void fillCompact(final ViewHolder h, final WeekSet a, boolean animate) {
                super.fillCompact(h, a, animate);

                updateTime(h, (ReminderSet) a);
                h.time.setClickable(false);

                h.every.setClickable(false);

                updateEvery(h, a);
            }
        };
        adapter.setHasStableIds(true);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_alarms, container, false);
        list = (RecyclerView) rootView.findViewById(R.id.section_label);
        list.setLayoutManager(new LinearLayoutManager(getContext()));
        list.setAdapter(adapter);
        list.setItemAnimator(animator);
        list.addOnScrollListener(animator.onScrollListener);
        list.addItemDecoration(new DividerItemDecoration(list.getContext(), DividerItemDecoration.VERTICAL));

        if (selected > 0)
            list.smoothScrollToPosition(adapter.getPosition(selected));

        return rootView;
    }

    @Override
    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
        super.onSharedPreferenceChanged(sharedPreferences, key);
        if (key.startsWith(HourlyApplication.PREFERENCE_REMINDERS_PREFIX)) {
            items.loadReminders(sharedPreferences);
            adapter.notifyDataSetChanged();
        }
    }

    @Override
    public void setEnable(WeekSet a, boolean e) {
        super.setEnable(a, e);
    }

    void save(WeekSet a) {
        super.save(a);
        SharedPreferences shared = PreferenceManager.getDefaultSharedPreferences(getActivity());
        shared.unregisterOnSharedPreferenceChangeListener(this); // prevent reload reminders
        items.saveReminders();
        shared.registerOnSharedPreferenceChangeListener(this);
        adapter.notifyItemChanged(items.reminders.indexOf(a));
    }

    public void addAlarm(ReminderSet a) {
        int pos = items.reminders.size();
        items.reminders.add(a);
        adapter.notifyItemInserted(pos);
        selectAdd(a.id);
        list.smoothScrollToPosition(pos);
        SharedPreferences shared = PreferenceManager.getDefaultSharedPreferences(getActivity());
        shared.unregisterOnSharedPreferenceChangeListener(this);
        items.saveReminders();
        shared.registerOnSharedPreferenceChangeListener(this);
        boxAnimate = false;
    }

    @Override
    public void remove(WeekSet a) {
        super.remove(a);
        items.reminders.remove(a);
        SharedPreferences shared = PreferenceManager.getDefaultSharedPreferences(getActivity());
        shared.unregisterOnSharedPreferenceChangeListener(this);
        items.saveReminders();
        shared.registerOnSharedPreferenceChangeListener(this);
    }

    void updateEvery(ViewHolder h, WeekSet a) {
        h.everyText.setText(everyText(getContext(), ((ReminderSet) a).repeat));
    }

    void updateTime(ViewHolder h, ReminderSet a) {
        h.time.setText(a.format());
    }

    @Override
    public void onDismiss(DialogInterface dialogInterface) {
        if (dialogInterface instanceof HoursDialogFragment.Result) {
            HoursDialogFragment.Result r = (HoursDialogFragment.Result) dialogInterface;
            if (!r.ok)
                return;
            if (r.index == -1) {
                addAlarm(r.rs);
            } else {
                ReminderSet rs = items.reminders.get(r.index);
                rs.load(r.rs);
                save(rs);
            }
        }
        if (dialogInterface instanceof RepeatDialogFragment.Result) {
            RepeatDialogFragment.Result r = (RepeatDialogFragment.Result) dialogInterface;
            if (!r.ok)
                return;
            ReminderSet rs = items.reminders.get(r.index);
            rs.repeat = r.mins;
            rs.reload(); // update hours
            save(rs);

            // it is only for 23 api phones and up. since only alarms can trigs often then 15 mins.
            if (Build.VERSION.SDK_INT >= 23) {
                if (rs.repeat > 0 && rs.repeat < 15) {
                    SharedPreferences shared = android.support.v7.preference.PreferenceManager.getDefaultSharedPreferences(getActivity());
                    boolean b = shared.getBoolean(HourlyApplication.PREFERENCE_ALARM, false);
                    if (!b) {
                        Toast.makeText(getActivity(), R.string.alarm_type_alarm, Toast.LENGTH_SHORT).show();
                        SharedPreferences.Editor edit = shared.edit();
                        edit.putBoolean(HourlyApplication.PREFERENCE_ALARM, true);
                        edit.commit();
                    }
                }
            }
        }
    }

    @Override
    public Uri fallbackUri(Uri uri) {
        if (uri != null)
            return uri;
        else
            return ReminderSet.DEFAULT_NOTIFICATION;
    }

    @Override
    public Sound.Silenced playPreview(WeekSet a) {
        Sound.Silenced s = sound.playReminder((ReminderSet) a, System.currentTimeMillis(), new Runnable() {
            @Override
            public void run() {
                previewCancel();
            }
        });
        Sound.silencedToast(getContext(), s, System.currentTimeMillis());
        return s;
    }

    @Override
    public void setWeek(WeekSet a, int week, boolean c) {
        super.setWeek(a, week, c);
        if (a.noDays()) {
            a.weekdaysCheck = true;
            a.setEveryday();
            a.setWeek(week, false);
        }
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    @Override
    public void setMenuVisibility(final boolean menuVisible) {
        super.setMenuVisibility(menuVisible);
        if (!menuVisible)
            return;
        Activity a = getActivity();
        if (a == null) {
            handler.post(new Runnable() {
                @Override
                public void run() {
                    setMenuVisibility(menuVisible);
                }
            });
            return;
        }
        FloatingActionButton fab = (FloatingActionButton) a.findViewById(R.id.fab);
        fab.setImageResource(R.drawable.plus);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                HoursDialogFragment dialog = new HoursDialogFragment();

                Bundle args = new Bundle();
                args.putInt("index", -1);
                args.putStringArrayList("hours", new ArrayList<>(new ReminderSet(getActivity()).hours));
                args.putInt("repeat", 60);

                dialog.setArguments(args);
                dialog.show(getFragmentManager(), "");
            }
        });
    }
}
